﻿--[[
Author: Mantrid
https://fs.mantrid.net/

Copyright: All rights reserved
]]

local modDir = g_currentModDirectory
local modName = g_currentModName

local baleSwaps = {
	{ -- Square bales
		-- { w=0.45, h=0.35, l=1.20, xml="bales/squarebales/squarebale120.xml", origXml="data/objects/squarebales/squarebale120.xml" },
		-- { w=1.20, h=0.90, l=1.80, xml="bales/squarebales/squarebale180.xml", origXml="data/objects/squarebales/squarebale180.xml" },
		-- { w=1.20, h=0.90, l=2.20, xml="bales/squarebales/squarebale220.xml", origXml="data/objects/squarebales/squarebale220.xml" },
		-- { w=1.20, h=0.90, l=2.40, xml="bales/squarebales/squarebale240.xml", origXml="data/objects/squarebales/squarebale240.xml" }
	},
	{ -- Round bales
		{ w=1.20, d=1.25, xml="assets/roundbale125.xml", origXml="data/objects/roundbales/roundbale125.xml" },
		{ w=1.2, d=1.50, xml="assets/roundbale150.xml", origXml="data/objects/roundbales/roundbale150.xml" },
		{ w=1.2, d=1.80, xml="assets/roundbale180.xml", origXml="data/objects/roundbales/roundbale180.xml" },
	}
}

BaleSwapper = {}

local BaleSwapper_mt = Class(BaleSwapper)

function BaleSwapper:new(modDirectory, modName, customMt)
	local self = setmetatable({}, customMt or BaleSwapper_mt)

    self.modDirectory = modDirectory
    self.modName = modName
	
	local modDescFile = loadXMLFile("modDesc", modDirectory .. "modDesc.xml")
	self.title = getXMLString(modDescFile, "modDesc.title.en")
	self.author = getXMLString(modDescFile, "modDesc.author")
	self.version = getXMLString(modDescFile, "modDesc.version")
	delete(modDescFile)

	self.BaleSwapper = BaleSwapper
	self.BaleSwapperMenu = BaleSwapper.BaleSwapperMenu
	BaleSwapper.BaleSwapperMenu = nil
	
	Logging.info("Loaded mod " .. self.title .. " v" .. self.version .. " by " .. self.author)
	return self
end

-- Swap over the xml filename to base-game files, so bales still work without the mod
function BaleSwapper:before_saveToXMLFile(xmlFile, key)
	if self.customEnvironment == modName then
		xmlFile:removeProperty(key .. "#modName")
	end
	
	for baleType = 1, 2 do
		for _, row in ipairs(baleSwaps[baleType]) do
			if self.xmlFilename == modDir .. row.xml then
				self.xmlFilename = row.origXml
				return
			end
		end
	end
end

-- Save our bale info on game save
function BaleSwapper:after_saveToXMLFile(xmlFile, key)
	-- Restore the xml filenames to our own
	for baleType = 1, 2 do
		for _, row in ipairs(baleSwaps[baleType]) do
			if self.xmlFilename == row.origXml then
				self.xmlFilename = modDir .. row.xml
				return
			end
		end
	end
end

-- Overwritten function, to return our bale xml
function BaleSwapper:insteadOf_getBaleXMLFilename(superFunc, fillTypeIndex, isRoundbale, width, height, length, diameter, customEnvironment)
	local baleIndex = self:getBaleIndex(fillTypeIndex, isRoundbale, width, height, length, diameter, customEnvironment)

	if baleIndex ~= nil then
		if isRoundbale then
			if baleSwaps[2] ~= nil then
				for i = 1, #baleSwaps[2] do
					if MathUtil.round(baleSwaps[2][i].w, 2) == MathUtil.round(width, 2) and MathUtil.round(baleSwaps[2][i].d, 2) == MathUtil.round(diameter, 2) then
						return modDir .. baleSwaps[2][i].xml, baleIndex
					end
				end
			end
		else
			if baleSwaps[1] ~= nil then
				for i = 1, #baleSwaps[1] do
					if MathUtil.round(baleSwaps[1][i].w, 2) == MathUtil.round(width, 2) and MathUtil.round(baleSwaps[1][i].l, 2) == MathUtil.round(length, 2) and MathUtil.round(baleSwaps[1][i].h, 2) == MathUtil.round(height, 2) then
						return modDir .. baleSwaps[1][i].xml, baleIndex
					end
				end
			end
		end

		return self.bales[baleIndex].xmlFilename, baleIndex
	end

	return nil
end

-- Store items and EasyDev bales
function BaleSwapper:before_loadFromConfigXML(superFunc, xmlFilename, x, y, z, rx, ry, rz)
	for baleType = 1, 2 do
		for _, row in ipairs(baleSwaps[baleType]) do
			if xmlFilename == row.origXml then
				xmlFilename = modDir .. row.xml
				return superFunc(self, xmlFilename, x, y, z, rx, ry, rz)
			end
		end
	end
	
	return superFunc(self, xmlFilename, x, y, z, rx, ry, rz)
end
